# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

package UML::Config;

use strict;

sub do_file {
    my $file = shift;
    my $ret = do $file;

    if(!$ret){
	if($@){
	    die "Couldn't parse '$file' : $@\n";
	}
	elsif(!defined($ret)){
	    die "Couldn't run '$file': $!\n";
	}
	else {
	    die "Couldn't run '$file' for some unknown reason\n";
	}
    }
    return($ret);
}

sub merge_option {
    my $to = shift;
    my $key = shift;
    my $val = shift;
    my @keys = split("/", $key);

    my $current = $to;
    my $last = pop(@keys);

    for my $segment (@keys){
	!defined($current->{$segment}) and $current->{$segment} = { };
	$current = $current->{$segment};
    }
    $current->{$last} = $val;
}

sub merge_options {
    my $from = shift;
    my $to = shift;

    foreach my $key (keys(%$from)){
	if((ref($from->{$key}) eq 'HASH') && defined($to->{$key})){
	    merge_options($from->{$key}, $to->{$key});
	}
	else {
	    $to->{$key} = $from->{$key};
	}
    }
}

sub canonicalize {
    my $from = shift;
    my $to = { };

    ref($from) ne "HASH" and return($from);

    foreach my $key (keys(%$from)){
	my $val = canonicalize($from->{$key});
	merge_option($to, $key, $val);
    }
    return($to);
}

sub new {
    my $class = shift;
    my $file = shift;
    my $local;

    my $me = canonicalize(do_file($file));

    defined($ENV{"HOME"}) and -r "$ENV{HOME}/.umltest" and 
	$local = canonicalize(do_file("$ENV{HOME}/.umltest"));
    defined($local) and merge_options($local, $me);

    foreach my $option (@_){
	my ($key, $value) = split("=", $option);
	merge_option($me, $key, $value);
    }

    bless($me, $class);
    return($me);
}

sub copy {
    my $old = shift;

    ref($old) ne 'HASH' and return($old);
    my $new = { };
    foreach my $option (keys(%$old)){
	$new->{$option} = copy($old->{option});
    }
    return($new);
}

sub clone {
    my $me = shift;
    my $new = copy($me);
    
    bless($new, ref($me));
    return($new);
}

sub sub_test {
    my $me = shift;
    my $test = shift;
    my $ret = { uml => $me->{uml}, 
		tests => $me->{tests}->{$test}, 
		filesystems => $me->{filesystems} };

    bless($ret, ref($me));
    return($ret);
}

sub as_hash {
    my $me = shift;

    return({ %$me });
}

sub uml_config {
    my $me = shift;
    my $filesystem = shift;
    my %values = %{$me->{uml}};

    !defined($filesystem) and defined($values{ubd}) and 
	$filesystem = $values{ubd}->{0};
	
    defined($filesystem) and
	merge_options($me->{roots}->{$filesystem}, \%values);

    return(%values);
}

sub test_option {
    my $me = shift;
    my $name = shift;

    return($me->{tests}->{$name});
}

sub add_defaults {
    my $me = shift;

    while(@_){
	my ($arg, $val) = (shift(), shift());
	$me->{uml}->{$arg} = $val;
    }
}

sub get_value {
    my $me = shift;
    my $key = shift;

    my @keys = split('/', $key);
    foreach my $option (@keys){
	!defined($me->{$option}) and return(undef);
	$me = $me->{$option};
    }
    return($me);
}

sub set_value {
    my $me = shift;
    my $key = shift;
    my $value = shift;

    $me->merge_option($key, $value);
}

1;
