# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

package UML::Log;

use IO::File;

use strict;

sub new {
    my $class = shift;
    my $file = shift;
    my $handle = new IO::File ">>$file" or die "Couldn't open $file : $!";
    $handle->autoflush(1);
    my $me = { handle => $handle };

    bless($me, $class);
    return($me);
}

sub logger {
    my $me = shift;
    my $handle = $me->{handle};

    return(sub { my $str = shift; print $handle $str; } );
}

sub print_log {
    my $me = shift;
    my $str = shift;
    my $stars = shift;
    my $format = shift;
    my $date = `date "+%D %T"`;
    my $handle = $me->{handle};
    my $spaces = " " x (5 - length($stars));
    
    chomp $date;
    print $handle "\n$stars$spaces [ $date ] $str $stars\n";
}

sub log_run {
    my $me = shift;
    my $str = shift;

    print_log($me, $str, "*****");
}

sub log_set {
    my $me = shift;
    my $str = shift;

    print_log($me, $str, "****");
}

sub log_test {
    my $me = shift;
    my $str = shift;

    print_log($me, $str, "***");
}

sub close {
    my $me = shift;

    close $me->{handle};
}

1;
