# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

package UML::MConsole;

use Socket;
use strict;

my $MCONSOLE_MAGIC = 0xcafebabe;
my $MCONSOLE_MAX_DATA = 512;
my $MCONSOLE_VERSION = 1;

sub new {
    my $class = shift;
    my $umid = shift;
    my $me = { socket => undef, target => undef };

    !defined($umid) and die "UML::MConsole::new - umid must be defined\n";
    my $socket;

    my $home = $ENV{HOME};
    if(defined($home)){
	-S "$home/.uml/$umid/mconsole" and 
	    $socket = "$home/.uml/$umid/mconsole";
    }
    if(!defined($socket) && (-S $umid)){
	$socket = $umid;
    }
    if(!defined($socket)) {
	die "UML::MConsole::new - couldn't find a Unix socket at either " .
	    "$home/.uml/$umid/mconsole or $umid\n";
    }

    $me->{target} = sockaddr_un($socket);
    !defined($me->{target}) and 
	die "UML::MConsole::new - sockaddr_un failed : $!\n";

    !defined(socket(SOCK, AF_UNIX, SOCK_DGRAM, 0)) and 
	die "UML::MConsole::new - socket failed : $!\n";
    $me->{socket} = \*SOCK;

# XXX the -u causes mktemp to unlink the file - this is needed because
# we don't want a normal file, we want a socket.

    my $file = `mktemp -u /tmp/mconsole.XXXXXX`;
    chomp $file;
    $? != 0 and die "UML::MConsole::new - mktemp failed with status $?\n";
    !defined(bind($me->{socket}, sockaddr_un($file))) and 
	die "UML::MConsole::new - bind failed : $!\n";

    bless($me, $class);
    return($me);
}

sub command {
    my $sock = shift;
    my $cmd = shift;
    my $where = shift;
    my $msg = pack("LiiA*", $MCONSOLE_MAGIC, $MCONSOLE_VERSION, length($cmd),
		   $cmd);

    !defined(send($sock, $msg, 0, $where)) and die "send failed : $!\n";
    my ($err, $more, $len);
    my $msg = "";
    do {
	my $cur = "";
	my $data = "";
	!defined(recv($sock, $data, $MCONSOLE_MAX_DATA, 0)) and 
	    die "recv failed : $!\n";
	($err, $more, $len, $cur) = unpack("iiiA*", $data);
	$err and die "mconsole command '$cmd' failed : $cur\n";
	$msg .= $cur;
    } while($more);
    return($msg);
}

sub version {
    my $me = shift;

    return(command($me->{socket}, "version", $me->{target}));
}

sub absolutize {
    my $file = shift;
    my $pwd = `pwd`;
    chomp $pwd;

    !defined($file) and return(undef);
    if($file =~ /(.*)\/([^\/]*)$/){
	my $dir = $1;
        $file = $2;
	chdir($dir) or die "Couldn't chdir to '$dir' : $!";
	$dir = `pwd`;
	chomp $dir;
	chdir($pwd);
	return("$dir/$file");
    }
    else {
	return("$pwd/$file");
    }
}

sub fix_filenames {
    my $file = shift;
    my $cow_file;
    my $backing_file = $file;

    if($file =~ /([^,]*),(.*)/){
	$cow_file = $1;
	$backing_file = $2;
    }

    $cow_file = absolutize($cow_file);
    $backing_file = absolutize($backing_file);

    if(defined($cow_file)){
	return("$cow_file,$backing_file");
    }
    else {
	return("$backing_file");
    }
}

sub config {
    my $me = shift;
    my $dev = shift;
    my $config = shift;

    $dev =~ /ubd.*/ and $config = fix_filenames($config);
    return(command($me->{socket}, "config $dev=$config", $me->{target}));
}

sub remove {
    my $me = shift;
    my $dev = shift;

    return(command($me->{socket}, "remove $dev", $me->{target}));
}

sub help {
    my $me = shift;

    return(command($me->{socket}, "help", $me->{target}));
}

sub reboot {
    my $me = shift;

    return(command($me->{socket}, "reboot", $me->{target}));
}

sub halt {
    my $me = shift;

    return(command($me->{socket}, "halt", $me->{target}));
}

sub sysrq {
    my $me = shift;
    my $op = shift;

    return(command($me->{socket}, "sysrq $op", $me->{target}));
}

sub cad {
    my $me = shift;

    return(command($me->{socket}, "cad", $me->{target}));
}

1;
