# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

package UML::Test;

use strict;

sub new {
    my $class = shift;
    my $me = { name => undef, 
	       requirements => { },
	       configure => undef,
	       kconfig => { },
	       config => { },
	       tester => undef,
	       time => 30
	     };

    while(@_){
	(my $key, my $val, @_) = @_;
	if($key eq "state"){
	    $me->{requirements}->{state} = $val;
	}
	elsif($key eq "run"){
	    $me->{tester} = $val;
	}
	elsif($key eq "configure"){
	    $me->{configure} = $val;
	}
	elsif($key eq "time"){
	    $me->{time} = $val;
	}
	elsif($key eq "kconfig"){
	    foreach my $option (keys %$val){
		$me->{kconfig}->{$option} = $val->{$option};
	    }
	}
	elsif($key eq "config"){
	    foreach my $option (keys(%$val)){
		$me->{config}->{$option} = $val->{$option};
	    }
	}
	else {
	    die "Bad argument : $key = $val";
	}
    }
    bless($me, $class);
    return($me);
}

sub name {
    my $me = shift;
    my $name = shift;

    !defined($name) and return($me->{name});
    $me->{name} = $name;
}

sub require {
    my $me = shift;
    my $what = shift;
    my $val = shift;

    !defined($val) and return($me->{requirements}->{$what});
    $me->{requirements}->{$what} = $val;
}

sub time_left {
    my $me = shift;
    my $now = time();

    !defined($me->{deadline}) and return(undef);
    if($now > $me->{deadline}){
	die "Time expired\n";
    }
    return($me->{deadline} - $now);
}

sub do {
    my $me = shift;
    my $uml = shift;
    my $log = shift;
    my $name = shift;
    my $options = shift;
    my $time = defined($options->test_option("time")) ? 
 	       $options->test_option("time") : $me->{time};

    $time != 0 and $me->{deadline} = time() + $time;
    $me->{tester}->($uml, $log, $name, $options);
}

sub config {
    my $me = shift;
    my $config = shift;

    !defined($me->{configure}) and return($me);
    return($me->{configure}->($me, $config));
}

sub kconfig {
    my $me = shift;
    my %new = @_;

    foreach my $option (keys(%new)){
	$me->{kconfig}->{$option} = $new{$option};
    }
    return($me->{kconfig});
}

sub run {
    my $me = shift;
    my $run = shift;

    $me->{tester} = $run;
}

sub config_uml {
    my $me = shift;
    my $config = shift;

    foreach my $option (keys(%{$me->{config}})){
	$config->set_value($option, $me->{config}->{$option});
    }
}

1;
