# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

package UML::Testlib;

use UML;

use Exporter   ();
use vars       qw(@ISA @EXPORT);

@ISA         = qw(Exporter);
@EXPORT      = qw(&run_exit_0 &compile &compile_run_exit_0
		  &compile_run_interact_exit_0 &interact_exit_0 &one_interact
		  &add_swap &remove_swap &gdb);

sub here_interact {
    my $expect = shift;
    my $time = shift;
    my $string = shift;

    my @lines = split('\n', $string);
    foreach my $line (@lines){
	$expect->print("$line\n");

	my ($pos, $err, $match, $before, $after) = 
	    $expect->expect($time, "\n", @_);
	$pos != 1 and return($pos - 1, $err, $match, $before, $after);
    }
    $expect->print("EOF\n");
    return($expect->expect($time, @_));
}

sub here_doc {
    my $uml = shift;
    my $file = shift;
    my $input = shift;

    $uml->command("cat > $file << EOF\n", 
		  sub { my $expect = shift;
			my $time = shift;
			here_interact($expect, $time, $input, @_); } );
}

sub run_exit_0 {
    my $uml = shift;

    my @outputs = ();
    foreach my $cmd (@_){
	my $output = $uml->command($cmd);
	push @outputs, $output;
	my $status = $uml->command('echo $?');
	!defined($status) and die "Couldn't figure out exit status\n";
	$status != 0 and die "Command '$cmd' exited with status $status\n";
    }
    return(@outputs);
}

sub compile {
    my ($uml, $prog, $cflags) = @_;
    my $exe = "./test";

    !defined($cflags) and $cflags = "";
    here_doc($uml, "$exe.c", $prog);
    run_exit_0($uml, "cc $cflags -o $exe $exe.c");
    return($exe);
}

sub compile_run_exit_0 {
    my ($uml, $prog, $cflags) = @_;

    my $exe = compile($uml, $prog, $cflags);
    run_exit_0($uml, $exe);
}

sub interact_exit_0 {
    my ($uml, $exe, $interactor) = @_;

    $uml->command($exe, $interactor);

    my $status = $uml->command('echo $?');
    !defined($status) and die "Couldn't figure out exit status\n";
    $status != 0 and die "Command exited with status $status\n";    
}    

sub compile_run_interact_exit_0 {
    my ($uml, $prog, $interactor) = @_;

    my $exe = compile($uml, $prog);
    interact_exit_0($uml, $exe, $interactor);
}

sub one_interact {
    my $expect = shift;
    my $time = shift;
    my $commands = shift;
    my $want = shift;

    ref($commands) ne "ARRAY" and $commands = [ $commands ];
    my @outputs = ();
    foreach my $command (@$commands){
	$expect->print("$command\n");
	my ($pos, $err, $match, $before, $after) = 
	    $expect->expect($time, $want, @_);
	$pos != 1 and die [ $pos - 1, $err, $match, $before, $after ];
	push @outputs, $before;
    }
    return(@outputs);
}

my $swap_num = 0;

sub add_swap {
    my $uml = shift;
    my $megs = shift;
    my $swap = "swap$swap_num";
    $swap_num++;
    my $dd = "dd if=/dev/zero of=$swap bs=1024 seek=\$(($megs * 1024)) " .
	"count=1";

    my $err = `$dd 2>&1`;
    if($? ne 0){
	unlink "$swap";
	die "Creating swap file failed : $err\n";
    }
    my $dev = $uml->add($swap, cow => 0);
    run_exit_0($uml, "mkswap /dev/ubd/$dev", "swapon /dev/ubd/$dev");
    return( [ $swap, $dev ] );
}

sub remove_swap {
    my $uml = shift;
    my $info = shift;
    my ($swap, $dev) = @$info;

    $uml->command("swapoff /dev/ubd/$dev");
    $uml->remove("$dev");
    unlink $swap;    
}

sub gdb_interact {
    my $interact = shift;
    my $expect = shift;
    my $time = shift;

    my ($pos, $err, $match, $before, $after) = 
	$expect->expect($time, "(gdb) ", @_);
    $pos != 1 and return($pos - 1, $err, $match, $before, $after);

    eval { $interact->($expect, $time) };
    my $fail = $@;

    $expect->print("quit\n");
    ($pos, $err, $match, $before, $after) = 
	$expect->expect($time, "Exit anyway? (y or n) ", @_);
    if($pos == 1){
	$expect->print("y\n");
	($pos, $err, $match, $before, $after) = $expect->expect($time, @_);
    }
    else {
	$pos--;
    }

    if($fail){
	ref($fail) eq "ARRAY" and return(@$err);
	die $fail;
    }	

    return($pos, $err, $match, $before, $after);    
}

sub gdb {
    my $uml = shift;
    my $exe = shift;
    my $interact = shift;

    interact_exit_0($uml, "gdb $exe", sub { gdb_interact($interact, @_) } );
}

1;
