#!/bin/bash

# 'status' is used for a timestamp.  If the latest patch is newer than it, 
# then it needs testing.  If 'lock' exists, then the test suite is in progress.
#
# This takes four arguments, the directory which contains the kernel tarballs,
# the directory in which it's supposed to look for new patches, the
# root of the test suite hierarchy, and the file to dump the summary in.

KERNEL=$1
UML=$2
TEST_ROOT=$3
SUMMARY=$4

STATUS=status
LOCK=lock
OUTPUT=tests.out

cd $TEST_ROOT

latest=`cd $UML ; ls -1t uml-patch-*.bz2 | head -1`
[ -e $STATUS -a $STATUS -nt $UML/$latest ] && exit 1
[ -e $LOCK ] && exit 1

touch $LOCK

version=`echo $latest | sed 's/uml-patch-\(.*\)-.*.bz2/\1/'`
kernel=$KERNEL/linux-$version.tar.bz2

rm -rf $TEST_ROOT/linux
bash -e build.sh $kernel $UML/$latest > build.out 2>&1
if [ $? -ne 0 ]
then
    echo 'The UML build failed :' > $SUMMARY
    rm -f $LOCK
    rm -rf linux
    cat build.out >> $SUMMARY
    echo "$version build failed" > $STATUS
    exit 0
fi

out=tests.out

perl test.pl -c kernel_pool=$TEST_ROOT/linux -o $out > $SUMMARY 2>&1

cd $TEST_ROOT

echo $version > $STATUS

rm -f $LOCK

exit 0
