use UML;
use UML::TestSuite;
use UML::Log;
use UML::Config;

use strict;

my @test_sets = ();
my @configs = ();

my $logfile = "tests.log";

sub Usage {
    print "Usage - perl test.pl [ -o log-file ] [ -c config-string ... ] " .
	"[ test-directory ...]\n";
    exit(1);
}

while(@ARGV){
    my $arg = shift;
    if($arg eq "-c"){
	!@ARGV and Usage();
	my $config = shift;
	push @configs, $config;
    }
    elsif($arg eq "-o"){
	!@ARGV and Usage();
	$logfile = shift;	
    }
    else {
	push @test_sets, "$arg";
    }
}

unlink $logfile;

my $config = UML::Config->new("Config.pl", @configs);
my $suite = UML::TestSuite->new("tests");
$suite->populate("tests", $config->as_hash(), @test_sets);

$config->add_defaults("con0" => "fd:0,fd:1", "cow_ubds" => 1);

my $log = UML::Log->new($logfile);

my ($start_time, $end_time, @results) = $suite->run($config, $log);

my $successes = 0;
my $failures = 0;

foreach my $result (@results){
    my ($name, $success, $elapsed, $failure_status) = @$result;
    if($success){
	$successes++;
    }
    else {
	$failures++;
    }
}

sub parse_time {
    my $secs = shift;

    return(sprintf("%02d:%02d", $secs / 60, $secs % 60));
}

my $summary = "\nResults : $successes successes, $failures failures\n";
$summary .= "Total time : " . parse_time($end_time - $start_time) . "\n\n";
foreach my $result (@results){
    my ($name, $success, $elapsed, $failure_status) = @$result;
    my $status = $success ? "succeeded" : "failed";
    $summary .= "\t$name (" . parse_time($elapsed) . ") : $status";
    $success eq 0 and $summary .= " - $failure_status";
    $summary .= "\n";
}

my $logger = $log->logger();
$logger->($summary);
$log->close();

print "$summary";

exit 0;
