# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

use UML::TestSuite;
use UML::Test;
use UML::Config;

use strict;

sub one_boot {
    my $filesystem = shift;
    my $uml = shift;
    my $log = shift;
    my $name = shift;
    my $config = shift;

    my $remove = 0;
    if($filesystem =~ /\.bz2$/){
	$log->log_test("Copying $filesystem and uncompressing it");
	my $err = `cp -f $filesystem . 2>&1`;
	if($? ne 0){
	    if($filesystem =~ /.*\/([^\/]*).bz2$/){
		unlink $1;
	    }
	    chomp $err;
	    die "'cp $filesystem .' failed : $err";
	}
	$filesystem =~ /.*\/([^\/]*).bz2$/ and $filesystem = $1;
	unlink $filesystem;
	$err = `bunzip2 -f $filesystem.bz2 2>&1`;
	if($? ne 0){
	    unlink $filesystem.bz2 $filesystem;
	    chomp $err;
	    die "'bunzip2 -f $filesystem.bz2' failed : $err";
	}
	$remove = 1;
    }
    $log->log_test("Booting $filesystem");
    $uml->args(ubd => { 0 => "$filesystem" } );
    eval {
	$uml->boot($log->logger());
	$uml->halt();
	$log->log_test("Halted $filesystem");
    };
    my $status = $@;
    $remove and unlink "$filesystem";
    $status and die $status;
}

sub config {
    my $test = shift;
    my $config = shift;
    my $patterns = $config->{filesystems};
    my %time = ();
    defined($config->{time_per_boot}) and 
	$time{time} = $config->{time_per_boot};

    ref($patterns) ne 'ARRAY' and $patterns = [ $patterns ];
    my @filesystems = ();
    foreach my $filesystem (@$patterns){
	push @filesystems, glob($filesystem);
    }

    my %tests = ();
    foreach my $filesystem (@filesystems){
	my $new = UML::Test->new(state => "down", 
				 run => sub { one_boot($filesystem, @_) }, 
				 %time);
	my @segments = split('/', $filesystem);
	my $short = pop @segments;
	$short =~ s/\.bz2$//;
	$new->name($short);
	$tests{$filesystem} = $new;
    }
    return(UML::TestSuite->new($test->name(), %tests));
}

UML::Test->new(state => "down", run => \&run, configure => \&config);
