# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

use UML::Testlib;
use UML::Test;

use strict;

my $prog = '
#include <unistd.h>
#include <stdio.h>

int main()
{
        printf("@@@%d@@@\n", getpid());
        printf("@@@%d@@@\n", getpid());
}
';

sub interact {
    my $expect = shift;
    my $time = shift;

    my ($disas) = one_interact($expect, $time, "disas getpid", "(gdb) ", @_);
    $disas !~ /(0x[0-9a-f]+)\s*<[^>]*getpid\+\d+>:\s*int\s+\$0x80/ and
	die "Couldn't figure out where syscall is\n";
    my $addr = $1;

    my @addresses = ($disas =~ /^(0x[0-9a-f]+)/mg);
    my $next;
    foreach my $i (0 .. $#addresses - 1){
	if($addresses[$i] eq $addr){
	    $next = $addresses[$i + 1];
	    last;
	}
    }
    !defined($next) and 
	die "Couldn't figure out what followed the system call\n";

    my $commands = [ "break main", 
		     "run",
		     "break *$addr",
		     "cont", 
		     "stepi",
		     "d 2", 
		     "cont" ];
    my @outputs = one_interact($expect, $time, $commands, "(gdb) ", @_);

    my $pids = $outputs[$#outputs];
    my ($pid1, $pid2) = ($pids =~ /@@@(\d+)@@@/g);
    (!defined($pid1) || !defined($pid2)) and 
	die "Couldn't extract pids from output\n";
    $pid1 != $pid2 and die "pids differ : $pid1 vs $pid2\n";

    my $after = $outputs[$#outputs - 2];
    my ($after_addr) = ($after =~ /^(0x[0-9a-f]+)/m);
    $after_addr ne $next and die "Stopped at '$after_addr', wanted '$next'\n";
}

sub run {
    my $uml = shift;

    my $exe = compile($uml, $prog, "-g -static");
    gdb($uml, "$exe", \&interact);
}

UML::Test->new(state => "up", run => \&run);
