# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

use UML::Testlib;
use UML::Test;

use strict;

my $prog = '
#include <stdio.h>

int main(int argc, char **argv)
{
        printf("Here I am\n");
        while(1) sleep(30);
}
';

sub bg_interact {
    my $expect = shift;
    my $time = shift;

    my ($pos, $err, $match, $before, $after) = 
	$expect->expect($time, "Here I am", @_);
    $pos != 1 and return($pos - 1, $err, $match, $before, $after);
    
    $expect->print(chr(26));
    return($expect->expect($time, @_));
}

sub gdb_interact {
    my $expect = shift;
    my $time = shift;

    my ($pos, $err, $match, $before, $after) = 
	$expect->expect($time, "(gdb) ", @_);
    $pos != 1 and return($pos - 1, $err, $match, $before, $after);

    my $bt;
    eval {
	($bt, undef) = one_interact($expect, $time, [ "bt", "det", ],
				    "(gdb) ", @_);
    };
    my $err = $@;

    $expect->print("quit\n");
    ($pos, $err, $match, $before, $after) = $expect->expect($time, @_);
    
    if($err){
	ref($err) eq "ARRAY" and return(@$err);
	die $err;
    }	

    return($pos, $err, $match, $bt, $after);
}

sub run {
    my $uml = shift;

    $uml->command("rm -rf core");
    $uml->command("ulimit -c 1024");
    my $exe = compile($uml, $prog, "-g");
    $uml->command("$exe", \&bg_interact);
    $uml->command("bg");
    my $pid = $uml->command("echo \$!");
    my $bt1 = $uml->command("gdb $exe $pid", \&gdb_interact);
    $uml->command("kill -11 $pid");
    my $bt2 = $uml->command("gdb $exe core", \&gdb_interact);
    $bt1 ne $bt2 and die "Process and core stacks don't match\n";
}

UML::Test->new(state => "up", run => \&run);
