# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#
use File::stat;
use UML::Test;
use UML::Testlib;

use strict;

sub run {
    my $image = shift;
    my $delete = shift;
    my $uml = shift;
    my $log = shift;
    my $name = shift;
    my $config = shift;

    $uml->args(initrd => $image, root => "/dev/ram");
    eval {
	$uml->boot($log->logger());
	$uml->halt();
    };
    my $status = $@;

    $delete and `rm -f $image`;
    $status and die $status;
}

sub initrd_size {
    my $goal = shift;
    my $size = shift;

    !defined($size) and return($goal);
    return($size >= $goal);
}

sub config {
    my $test = shift;
    my $config = shift;
    my $image = $config->{image};
    my $delete = 0;

    if(!defined($image)){
	warn("initrd : image not defined - skipping test\n");
	return(undef);
    }

    my $sb = stat($image);
    my $blocks = int(($sb->size / 1024) + 1);

    if($image !~ /.*\.gz$/){
	my $base = `basename $image`;
	chomp $base;
	my $output = `cp $image . ; gzip -f $base`;
	$? ne 0 and 
	    die "Copying and compressing initrd image failed : '$output'\n";
	$image = "$base.gz";
	$delete = 1;
    }

    $test->run(sub { run($image, $delete, @_) });
    $test->kconfig(BLK_DEV_RAM_SIZE => 
		   sub { return(initrd_size($blocks, @_)) });
    return($test);
}

UML::Test->new(state => "down", configure => \&config);
