# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

use UML::Test;
use UML::Testlib;

use strict;

my $iomem = '
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <asm/page.h>

int main(int argc, char *argv[])
{
  int fd, i, size = PAGE_SIZE;
  char *buf;

  if((fd = open("/dev/mmapper", O_RDWR)) < 0){
    perror("open");
    exit(1);
  }
  
  if((buf = mmap(NULL, size, PROT_WRITE | PROT_READ, MAP_SHARED,
                 fd, 0)) == MAP_FAILED){
    perror("mmap");
    exit(1);
  }
  for(i = 0; i < 1024; i++){
    if(buf[i] != \'0\') exit(1);
  }
      
  for(i = 1024; i < size; i++){
    if(buf[i] != \'\\0\') exit(1);
  }

  if(munmap(buf, size) < 0){
      perror("munmap");
      exit(1);
  }
  
  if((buf = mmap(NULL, 2 * size, PROT_WRITE | PROT_READ, MAP_SHARED,
                 fd, 0)) == MAP_FAILED){
      if(errno == EFAULT) exit(0);
  }

  exit(1);
}
';

my $pwd = `pwd`;
chomp $pwd;
my $file = "$pwd/iomem";

sub run {
    my $uml = shift;

    compile_run_exit_0($uml, $iomem);
    unlink $file;
}

my $output = `i=0; 
              while [ \$i -lt 1024 ]
              do
                  echo -n '0'; 
                  i=\$((\$i + 1)); 
              done > $file`;
$? ne 0 and die "Couldn't create $file : '$output'";

UML::Test->new(state => "up", 
	       run => \&run, 
	       kconfig => { MMAPPER => "y" },
	       config => { "uml/iomem" => "mmapper,$file" } );
