# 
# Copyright (C) 2002 Jeff Dike (jdike@karaya.com)
# Licensed under the GPL
#

use UML::Test;
use UML::Testlib;

use strict;

my $altstack = '#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <sys/mman.h>
#include <asm/page.h>

void *stack = NULL;

void handler(int sig)
{
  unsigned long stack_page, page;

  stack_page = ((unsigned long) stack) & ~(PAGE_SIZE - 1);
  page = ((unsigned long) &sig) & ~(PAGE_SIZE - 1);
  if(page != stack_page) exit(1);
}

int main(int argc, char **argv)
{
  stack_t ss;
  struct sigaction sa;

  stack = &ss;

  sa.sa_handler = handler;
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = 0;
  if(sigaction(SIGUSR1, &sa, NULL) < 0){
    perror("sigaction");
    exit(1);
  }
  kill(getpid(), SIGUSR1);

  stack = mmap(NULL, PAGE_SIZE, PROT_READ | PROT_WRITE | PROT_EXEC, 
	      MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
  if(stack < 0){
    perror("mmap");
    exit(1);
  }
  
  ss.ss_sp = stack;
  ss.ss_flags = 0;
  ss.ss_size = PAGE_SIZE;
  if(sigaltstack(&ss, NULL) < 0){
    perror("sigaltstack");
    exit(1);
  }

  sa.sa_handler = handler;
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = SA_ONSTACK;
  if(sigaction(SIGUSR1, &sa, NULL) < 0){
    perror("sigaction");
    exit(1);
  }
  kill(getpid(), SIGUSR1);
  exit(0);
}
';

sub run {
    my $uml = shift;

    compile_run_exit_0($uml, $altstack);
}

UML::Test->new(state => "up", run => \&run);
